'
' Clear $1 pool.
'
FUNCTION $1_Clear ()
	SHARED $1_pool$[]
	SHARED $1_poolUM[]

	FOR slot = UBOUND ($1_poolUM[]) TO 0 STEP -1
		$1_pool$[slot] = ""
		$1_poolUM[slot] = $$FALSE
	NEXT slot
END FUNCTION
'
' Deletes an item from $1 pool.
' $1_id = id of the item to delete
' RETURNs bOK: $$TRUE on success.
'
' Usage:
' bOK = $1_Delete ($1_id)
' IFF bOK THEN
'  PRINT "$1_Delete: Can't delete $1 item id "; $1_id
' ENDIF
'
FUNCTION $1_Delete ($1_id)
	SHARED $1_pool$[]
	SHARED $1_poolUM[]

	bOK = $$FALSE
	slot = $1_id - 1
	IF (slot >= 0) && (slot <= UBOUND ($1_poolUM[])) THEN
		$1_pool$[slot] = ""
		$1_poolUM[slot] = $$FALSE
		bOK = $$TRUE
	ENDIF
	RETURN bOK
END FUNCTION
'
' Gets $1 item from the $1 pool.
' $1_id = id of the item to get
' $1_item$ = RETURNed item
' RETURNs bOK: $$TRUE on success.
'
' Usage:
'	bOK = $1_Get ($1_id, @$1_item$)
'	IFF bOK THEN
'		PRINT "$1_Get: Can't get $1 item id "; $1_id
'	ENDIF
'
FUNCTION $1_Get ($1_id, @$1_item$)
	SHARED $1_pool$[]
	SHARED $1_poolUM[]

	bOK = $$FALSE
	slot = $1_id - 1
	IF (slot >= 0) && (slot <= UBOUND ($1_poolUM[])) THEN
		IF $1_poolUM[slot] THEN
			$1_item$ = $1_pool$[slot]
			bOK = $$TRUE
		ENDIF
	ENDIF
	IFF bOK THEN
		$1_item$ = ""
	ENDIF
	RETURN bOK
END FUNCTION
'
' Gets upper $1 item id.
'
FUNCTION $1_Get_idMax ()
	SHARED $1_poolUM[]

	idMax = 0
	IF $1_poolUM[] THEN
		FOR slot = UBOUND ($1_poolUM[]) TO 0 STEP -1
			IF $1_poolUM[slot] THEN
				idMax = slot + 1
				EXIT FOR
			ENDIF
		NEXT slot
	ENDIF
	RETURN idMax
END FUNCTION
'
' Initializes the $1 pool.
'
FUNCTION $1_Init ()
	SHARED $1_pool$[]		' an array of $1 items
	SHARED $1_poolUM[]		' usage map so we can see which $1_pool$[] elements are in use

	IFZ $1_pool$[] THEN
		DIM $1_pool$[7]
		DIM $1_poolUM[7]
	ENDIF
	FOR slot = UBOUND ($1_poolUM[]) TO 0 STEP -1
		$1_pool$[slot] = ""
		$1_poolUM[slot] = $$FALSE
	NEXT slot
END FUNCTION
'
' Adds a new item to the $1 pool.
' RETURNs the new $1 item id or 0 on fail.
'
' Usage:
' $1_id = $1_New ($1_item$)
' IFZ $1_id THEN
'  msg$ = "$1_New: Can't add new $1 item id " + STRING$ ($1_id)
'  PRINT msg$
' ENDIF
'
FUNCTION $1_New ($1_item$)
	SHARED $1_pool$[]
	SHARED $1_poolUM[]

	IFZ $1_poolUM[] THEN $1_Init ()

	slotNew = -1

	upper_slot = UBOUND ($1_poolUM[])
	FOR slot = 0 TO upper_slot
		IFF $1_poolUM[slot] THEN
			' reuse this open slot
			slotNew = slot
			EXIT FOR
		ENDIF
	NEXT slot

	IF (slotNew < 0) THEN
		' no room at the inn => add a new slot
		slotNew = upper_slot + 1
		'
		' expand $1_pool$[]
		upp = (slotNew << 1) | 3
		REDIM $1_pool$[upp]
		REDIM $1_poolUM[upp]
	ENDIF

	IF (slotNew >= 0) THEN
		$1_pool$[slotNew] = $1_item$
		$1_poolUM[slotNew] = $$TRUE
	ENDIF
	RETURN (slotNew + 1)
END FUNCTION
'
' Updates $1 item.
' $1_id = id of the item to update
' $1_item$ = updated item
' RETURNs bOK: $$TRUE on success.
'
' Usage:
' bOK = $1_Update ($1_id, $1_item$)
' IFF bOK THEN
'  PRINT "$1_Update: Can't update item id " + STRING$ ($1_id)
' ENDIF
'
FUNCTION $1_Update ($1_id, $1_item$)
	SHARED $1_pool$[]
	SHARED $1_poolUM[]

	bOK = $$FALSE
	slot = $1_id - 1
	IF (slot >= 0) && (slot <= UBOUND ($1_poolUM[])) THEN
		IF $1_poolUM[slot] THEN
			$1_pool$[slot] = $1_item$
			bOK = $$TRUE
		ENDIF
	ENDIF
	RETURN bOK
END FUNCTION